"""
    pdm-venv

    A plugin for pdm that enables virtualenv management
    :author: Frost Ming <mianghong@gmail.com>
    :license: MIT
"""

from typing import List

__all__: List[str] = []  # noqa: WPS410 (the only __variable__ we use)
